@@ -8,21 +8,23 @@ use Carp ();
 use Devel::Symdump;
 use Digest::MD5;
 use Graph::Easy;
-use List::MoreUtils  ();
-use Sub::Information ();
+use List::MoreUtils ();
+use Sub::Identify   ();
 use Text::SimpleTable;
 
+use constant PSEUDO_PACKAGES => qr/::(?:SUPER|ISA::CACHE)$/;
+
 =head1 NAME
 
 Class::Sniff - Look for class composition code smells
 
 =head1 VERSION
 
-Version 0.08
+Version 0.09
 
 =cut
 
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
 =head1 SYNOPSIS
 
@@ -106,8 +108,18 @@ Optional.
 If present and true, will attempt to include the C<UNIVERSAL> base class.  If
 a class hierarchy is pruned with C<ignore>, C<UNIVERSAL> may not show up.
 
+=item * clean
+
+Optional.
+
+If present, will automatically ignore "pseudo-packages" such as those ending
+in C<::SUPER> and C<::ISA::CACHE>.  If you have legitimate packages with these
+names, oops.
+
 =item * method_length
 
+Optional.
+
 If present, will set the "maximum length" of a method before it's reported as
 a code smell.  This feature is I<highly> experimental.  See C<long_methods>
 for details.
@@ -126,17 +138,18 @@ sub new {
     }
     my $self = bless {
         classes       => {},
+        clean         => $arg_for->{clean},
         duplicates    => {},
         exported      => {},
         graph         => undef,
         ignore        => $arg_for->{ignore},
         list_classes  => [$target_class],
         long_methods  => {},
+        method_length => ( $arg_for->{method_length} || 50 ),
         methods       => {},
         paths         => [ [$target_class] ],
         target        => $target_class,
         universal     => $arg_for->{universal},
-        method_length => ( $arg_for->{method_length} || 50 ),
     } => $class;
     $self->_initialize;
     return $self;
@@ -144,6 +157,9 @@ sub new {
 
 =head2 C<new_from_namespace>
 
+B<Warning>:  This can be a very slow method as it needs to exhaustively walk
+and analyze the symbol table.
+
  my @sniffs = Class::Sniff->new_from_namespace({
      namespace => $some_root_namespace,
      universal => 1,
@@ -165,6 +181,47 @@ sub new {
 Given a namespace, returns a list of C<Class::Sniff> objects namespaces which
 start with the C<$namespace> string.  Requires a C<namespace> argument.
 
+If you prefer, you can pass C<namespace> a regex and it will simply return a
+list of all namespaces matching that regex:
+
+ my @sniffs = Class::Sniff->new_from_namespace({
+     namespace => qr/Result(?:Set|Source)/,
+ });
+
+You can also use this to slurp "everything":
+
+ my @sniffs = Class::Sniff->new_from_namespace({
+     namespace => qr/./,
+     universal => 1,
+ });
+
+Note that because we still pull parents, it's possible that a parent class
+will have a namespace not matching what you are expecting.
+
+ use Class::Sniff;
+ use HTML::TokeParser::Simple;
+ my @sniffs = Class::Sniff->new_from_namespace({
+     namespace => qr/(?i:tag)/,
+ });
+ my $graph    = $sniffs[0]->combine_graphs( @sniffs[ 1 .. $#sniffs ] );
+ print $graph->as_ascii;
+ __END__
+ +-------------------------------------------+
+ |      HTML::TokeParser::Simple::Token      |
+ +-------------------------------------------+
+   ^
+   |
+   |
+ +-------------------------------------------+     +---------------------------------------------+
+ |   HTML::TokeParser::Simple::Token::Tag    | <-- | HTML::TokeParser::Simple::Token::Tag::Start |
+ +-------------------------------------------+     +---------------------------------------------+
+   ^
+   |
+   |
+ +-------------------------------------------+
+ | HTML::TokeParser::Simple::Token::Tag::End |
+ +-------------------------------------------+
+
 All other arguments are passed to the C<Class::Sniff> constructor.
 
 =cut
@@ -173,23 +230,64 @@ sub new_from_namespace {
     my ( $class, $arg_for ) = @_;
     my $namespace = delete $arg_for->{namespace}
       or Carp::croak("new_from_namespace requires a 'namespace' argument");
+    my $ignore = delete $arg_for->{ignore};
+
+    $namespace = ('Regexp' eq ref $namespace) 
+        ? $namespace
+        : qr/^$namespace/;
+
+    if (defined $ignore) {
+        $ignore = ('Regexp' eq ref $ignore) 
+            ? $ignore
+            : qr/^$ignore/;
+    }
+
     my @sniffs;
     my %seen;
-    my $new_sniff = sub {
+    my $find_classes = sub {
         my $symbol_name = shift;
         no warnings 'numeric';
         return if $seen{$symbol_name}++;    # prevent infinite loops
-        if ( $symbol_name =~ /^$namespace/ ) {
+        if ( $symbol_name =~ $namespace ) {
+            return if defined $ignore && $symbol_name =~ $ignore;
             $symbol_name =~ s/::$//;
             $arg_for->{class} = $symbol_name;
+            if ( not $class->_is_real_package($symbol_name) ) {
+                # we don't want to create a sniff, but we need to be able to
+                # descend into the namespace.
+                return 1;
+            }
             push @sniffs => Class::Sniff->new($arg_for);
         }
         return 1;
     };
-    B::walksymtable( \%::, 'NAME', $new_sniff );
+    B::walksymtable( \%::, 'NAME', $find_classes );
     return @sniffs;
 }
 
+=head2 C<graph_from_namespace>
+
+    my $graph = Class::Sniff->graph_from_namespace({
+        namespace => qr/^My::Namespace/,
+    });
+    print $graph->as_ascii;
+    my $graphviz = $graph->as_graphviz();
+    open my $DOT, '|dot -Tpng -o graph.png' or die("Cannot open pipe to dot: $!");
+    print $DOT $graphviz;
+
+Like C<new_from_namespace>, but returns a single C<Graph::Easy> object.
+
+=cut
+
+sub graph_from_namespace {
+    my ( $class, $arg_for ) = @_;
+    my @sniffs = $class->new_from_namespace($arg_for);
+    my $sniff  = pop @sniffs;
+    return @sniffs
+      ? $sniff->combine_graphs(@sniffs)
+      : $sniff->graph;
+}
+
 sub _initialize {
     my $self         = shift;
     my $target_class = $self->target_class;
@@ -226,9 +324,9 @@ sub _register_class {
     foreach my $method (@methods) {
         my $coderef = $class->can($method)
           or Carp::croak("Panic: $class->can($method) returned false!");
-        my $info = Sub::Information::inspect($coderef);
-        if ( $info->package ne $class ) {
-            $self->{exported}{$class}{$method} = $info->package;
+        my $package =  Sub::Identify::stash_name($coderef);
+        if ( $package ne $class ) {
+            $self->{exported}{$class}{$method} = $package;
         }
         else {
 
@@ -236,7 +334,7 @@ sub _register_class {
             # tricky and this is documented as experimental.
             local $@;
             eval {
-                my $line   = $info->line;
+                my $line   = B::svref_2object($coderef)->START->line;
                 my $length = B::svref_2object($coderef)->GV->LINE - $line;
                 if ( $length > $self->method_length ) {
                     $self->{long_methods}{"$class\::$method"} = $length;
@@ -525,9 +623,6 @@ sub duplicate_methods {
 Returns methods longer than C<method_length>.  This value defaults to 50 and
 can be overridden in the constructor (but not later).
 
-Caveats:  this is experimental and depends on C<Sub::Information> 0.10 and the
-ill-documented C<B> modules.  Specifically, it relies on the following:
-
 =over 4
 
 =item * How to count the length of a method.
@@ -569,7 +664,13 @@ Let me know how it works out :)
 
 =head3 Code Smell:  long methods
 
-Long methods are probably doing to much and should be broken down into smaller
+Note that long methods may not be a code smell at all.  The research in the
+topic suggests that methods longer than many experienced programmers are
+comfortable with are, nonetheless, easy to write, understand, and maintain.
+Take this with a grain of salt.  See the book "Code Complete 2" by Microsoft
+Press for more information on the research.  That being said ...
+
+Long methods might be doing to much and should be broken down into smaller
 methods.  They're harder to follow, harder to debug, and if they're doing more
 than one thing, you might find that you need that functionality elsewhere, but
 now it's tightly coupled to the long method's behavior.  As always, use your
@@ -935,6 +1036,15 @@ far in the hierarchy, the 'UNIVERSAL' class will not be added.
 
 sub universal { $_[0]->{universal} }
 
+=head2 C<clean>
+
+Returns true if user requested 'clean' classes.  This attempts to remove
+spurious packages from the inheritance tree.
+
+=cut
+
+sub clean     { $_[0]->{clean} }
+
 =head2 C<classes>
 
  my $num_classes = $sniff->classes;
@@ -998,7 +1108,7 @@ sub methods {
 
 sub _get_parents {
     my ( $self, $class ) = @_;
-    return if $class eq 'UNIVERSAL';
+    return if $class eq 'UNIVERSAL' or !$self->_is_real_package($class);
     no strict 'refs';
 
     my @parents = List::MoreUtils::uniq( @{"$class\::ISA"} );
@@ -1011,12 +1121,29 @@ sub _get_parents {
     return @parents;
 }
 
+sub _is_real_package {
+    my ( $proto, $class ) = @_;
+    no strict 'refs';
+    no warnings 'uninitialized';
+    return 1 if 'UNIVERSAL' eq $class;
+    return
+      unless eval {
+        defined *{ ${"${class}::"}{ISA} }{ARRAY}
+          || scalar grep { defined *{$_}{CODE} } values %{"$class\::"};
+      };
+}
+
 # This is the heart of where we set just about everything up.
 sub _build_hierarchy {
     my ( $self, @classes ) = @_;
-
     for my $class (@classes) {
-        return unless my @parents = $self->_get_parents($class);
+        if ( my $ignore = $self->ignore ) {
+            next if $class =~ $ignore;
+        }
+        if ( $self->clean ) {
+            next if $class =~ PSEUDO_PACKAGES;
+        }
+        next  unless my @parents = $self->_get_parents($class);
         $self->_register_class($_) foreach $class, @parents;
         $self->_add_children($class);
         $self->_build_paths($class);
@@ -1075,6 +1202,7 @@ sub _add_children {
     my @parents = $self->_get_parents($class);
 
     $self->{classes}{$class}{parents} = \@parents;
+
     foreach my $parent (@parents) {
         $self->_add_child( $parent, $class );
         $self->graph->add_edge_once( $class, $parent );
@@ -1099,7 +1227,7 @@ sub _add_child {
 
 User-defined package variables in OO code are a code smell, but with versions
 of Perl < 5.10, any subroutine also creates a scalar glob entry of the same
-name, so I've no done a package variable check yet.  This will happen in the
+name, so I've not done a package variable check yet.  This will happen in the
 future (there will be exceptions, such as with @ISA).
 
 =item * C3 Support
@@ -1115,9 +1243,11 @@ Curtis "Ovid" Poe, C<< <ovid at cpan.org> >>
 
 =head1 BUGS
 
-Please report any bugs or feature requests to C<bug-class-sniff at rt.cpan.org>, or through
-the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Class-Sniff>.  I will be notified, and then you'll
-automatically be notified of progress on your bug as I make changes.
+Please report any bugs or feature requests to C<bug-class-sniff at
+rt.cpan.org>, or through the web interface at
+L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Class-Sniff>.  I will be
+notified, and then you'll automatically be notified of progress on your bug as
+I make changes.
 
 =head1 SUPPORT
 
